/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.util.*;
import org.apache.log4j.*;
import org.w3c.dom.*;
import com.silverwrist.util.*;
import com.silverwrist.venice.except.*;
import com.silverwrist.venice.security.*;
import com.silverwrist.venice.svc.*;
import com.silverwrist.venice.util.XMLLoader;

class ServiceControlManager
{
  /*--------------------------------------------------------------------------------
   * Internal "Community Service Site" class
   *--------------------------------------------------------------------------------
   */

  class MyCommServiceSite implements CommServiceSite
  {
    MyCommServiceSite()
    { // do nothing
    } // end constructor

  } // end class MyCommServiceSite

  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(ServiceControlManager.class);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private Map comm_symbol_to_service;
  private Map comm_index_to_service;

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  ServiceControlManager(Document cfg, SecurityMonitor sm_global, SecurityMonitor sm_comm)
      throws ConfigException
  {
    XMLLoader loader = XMLLoader.get();
    Element root = loader.configGetRootElement(cfg,"services-config");
    DOMElementHelper root_h = new DOMElementHelper(root);
    NodeList nl;
    int i;

    // Get the "community" section
    Element sect = root_h.getSubElement("community");
    if (sect!=null)
    { // load the services for the section
      HashMap tmp_symbol_to_service = new HashMap();
      HashMap tmp_index_to_service = new HashMap();
      nl = sect.getChildNodes();
      for (i=0; i<nl.getLength(); i++)
      { // get each element node...
	Node n = nl.item(i);
	if ((n.getNodeType()==Node.ELEMENT_NODE) && (n.getNodeName().equals("service")))
	{ // create the appropriate service settings
	  CommService svc = initCommunityService((Element)n,sm_comm);
	  tmp_symbol_to_service.put(svc.getSymbol(),svc);
	  tmp_index_to_service.put(new Integer(svc.getIndex()),svc);

	} // end if

      } // end for

      if (tmp_symbol_to_service.size()>0)
	comm_symbol_to_service = Collections.unmodifiableMap(tmp_symbol_to_service);
      else
	comm_symbol_to_service = Collections.EMPTY_MAP;
      if (tmp_index_to_service.size()>0)
	comm_index_to_service = Collections.unmodifiableMap(tmp_index_to_service);
      else
	comm_index_to_service = Collections.EMPTY_MAP;

    } // end if
    else
    { // no community services - undo this
      comm_symbol_to_service = Collections.EMPTY_MAP;
      comm_index_to_service = Collections.EMPTY_MAP;

    } // end else

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private CommService initCommunityService(Element root, SecurityMonitor sm) throws ConfigException
  {
    CommService svc;
    // FUTURE: do a create of a specific class
    svc = new StaticCommService();
    svc.initialize(root,sm,new MyCommServiceSite());
    return svc;

  } // end initCommunityService

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public final void shutdown()
  {
    Iterator it = comm_symbol_to_service.values().iterator();
    while (it.hasNext())
    { // shut down each service in turn
      CommService svc = (CommService)(it.next());
      svc.shutdown();

    } // end while

  } // end shutdown

} // end class ServiceControlManager
